//------------------------------------------------------------------------------
// \file    test_audio.c
// \brief   implementation of OMAP-L138 audio test.
//
//-----------------------------------------------------------------------------

#include "stdio.h"
#include "types.h"
#include "evmomapl138.h"
//#include "evmomapl138_mcasp.h"
//#include "evmomapl138_aic3106.h"
//#include "test_audio.h"

//-----------------------------------------------------------------------------
// Private Defines and Macros
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Static Variable Declarations
//-----------------------------------------------------------------------------

static unsigned short sinetable[48] = {
    0x0000, 0x10b4, 0x2120, 0x30fb, 0x3fff, 0x4dea, 0x5a81, 0x658b,
    0x6ed8, 0x763f, 0x7ba1, 0x7ee5, 0x7ffd, 0x7ee5, 0x7ba1, 0x76ef,
    0x6ed8, 0x658b, 0x5a81, 0x4dea, 0x3fff, 0x30fb, 0x2120, 0x10b4,
    0x0000, 0xef4c, 0xdee0, 0xcf06, 0xc002, 0xb216, 0xa57f, 0x9a75,
    0x9128, 0x89c1, 0x845f, 0x811b, 0x8002, 0x811b, 0x845f, 0x89c1,
    0x9128, 0x9a76, 0xa57f, 0xb216, 0xc002, 0xcf06, 0xdee0, 0xef4c
};

//-----------------------------------------------------------------------------
// Private Function Prototypes
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Public Function Definitions
//-----------------------------------------------------------------------------

//define McBSP0 register
#define PCR  *(unsigned int *)0x01D10024
#define SRGR *(unsigned int *)0x01D10014
#define XCR  *(unsigned int *)0x01D10010
#define RCR  *(unsigned int *)0x01D1000C
#define SPCR *(unsigned int *)0x01D10008
#define DXR  *(unsigned int *)0x01D10004
#define DRR  *(unsigned int *)0x01D10000

void delay(unsigned int count)
{
	unsigned int i=0;
	for(i=0; i++; i<count)
	asm (" nop");
}
//-----------------------------------------------------------------------------
// \brief   tests the McBSP interface
//
// \param   none.
//
// \return  uint32_t
//-----------------------------------------------------------------------------
void TEST_McBSP(void)
{
   printf("--------------------------------------------------------------------\r\n");
   printf("                    McBSP test\r\n\r\n");
   
   printf("Because there is no Codec connected to McBSP on EVM\r\n");
   printf("--------------------\r\n");
   printf("So configure to use internal FS, bit CLK\r\n\r\n");
   printf("There is no real Data input or Output\r\n\r\n");
   
   printf("--------------------------------------------------------------------\r\n\r\n");
  
  // Enable PSC and config PINMUX for McBSP.
   printf("Enable McBSP0 via PSC, Disable first, Then Enable to Ensure McBSP in default state\r\n");
//   EVMOMAPL138_lpscTransition(PSC1, DOMAIN0, LPSC_MCBSP0, PSC_DISABLE);
   EVMOMAPL138_lpscTransition(PSC1, DOMAIN0, LPSC_MCBSP0, PSC_ENABLE);

   printf("Configure McBSP0 PINMUX\r\n");
   *(unsigned int *)0x01C14128 = 0x22222220;  //   PINMUX2 resgister

   printf("Initialize McBSP0 registers\r\n");  
   printf("For SPCR register configuration, MUST follow TRM McBSP Initialization Procedue!\r\n");  
   SPCR |=(1<<25|1<<24);
   SPCR &= ~(1<<23 |1<<22 | 1<<16 |1); // Disable FRST, GRST, XRST, RRST first.
	

   PCR  = (1 <<11 |  //FSXM FS is determined by FSGM in SRGR
           1 <<10 |   // FSRM  FS is generated internally, FSR is output
           1 << 9 |   // CLKXM is output pin driven by internal
           1 << 8 |   // CLKRM CLKR is output driven by internal
           0 << 7 |   // SCLKME  internal clock
           0 << 3 |   // FSXP    TX FS is active high
           0 << 2 |   // FSRP    RX FS in active high
           0 << 1 |   // CLKXP  TX data driven on rising edge of CLKX
           0 );       // CLKRP  RX data driven on falling edge of CLKR
 
   SRGR = (0 << 31 |  // sample rate generator clock is free running
   	       0 << 30 |  //Rising edge CLKS generates CLKG and FSG
   	       1 << 29 |  // smaple rate generator clock taken from internal 
   	       1 << 28 |  // FSX is driven by FSG.
   	       31<< 16 |  // Frame period value +1, 2 phase *16 bit.
   	       0 << 8  |  // Frame width value +1
   	      149<< 0 );   // sample rate generator clock frequency: CPU=300MHz, McBSP default to SYSCLK2=CPU/2, so bit clock=1MHz
   	                   // user need to configure it according to your own system requirement.
   	        
   RCR = ( 1 << 31  | // dual-phase frame
  	       0 << 24  |  //1 word in phase 2
  	       2 << 21  |  // 16 bit word in phase 2
  	       0 << 19  |  // No companding
  	       1 << 18  |  // Ignor extra fs pulse
  	       0 << 16  |  // 0-bit delay
  	       0 << 8   |  // 1 word in phase 1
  	       2 << 5   |  //16 bit word in phase 1
  	       0 << 4   |  // no reversal
  	       0   );
  
   XCR = ( 1 << 31   |  // dual-phase frame
  	       0 << 24   |  // 1 word in phase 2
  	       2 << 21   | // word length is 16 bit
  	       0 << 19   | //no companding
  	       1 << 18   |  // Ignore fs glitch
  	       0 << 16   |  //0 bit delay
  	       0 << 8    | //1 word in phase 1
  	       2 << 5    |  //16 bit word
  	       0 << 4    |  // no reversal
  	       0 );
  // 4. wait for internal synchronization.
  delay (100);
  // 5.a start the sample rate generator
  SPCR |= (1 << 22); // Enable GRST

  // 6.a, clear any potential XSYNCERR: enable transmitter
  SPCR |= (1 << 16); //Enabel XRST
  // 6.b.
  delay(50);
  SPCR &= ~(1 << 16); //Disable XRST

  //7.b initialize interrupt.
  InitDspIntc(); 
  EDMACC0_init();  
  //8. enable transimit and / or receive
  SPCR |=  (1 << 16 | 1); //Enable RRST, XRST
  //10. 
  if (!(SPCR & (1<<18)))  // Feed DXR if XSR is empty.
  	DXR =0xaa55;	

  //11. start internal frame sync generator 	         
   SPCR |= (1 << 23); //Enable FRST

}


unsigned int tempRX = 0x5555;

// set breakpoint within ISR to verify interrupt.
interrupt void McBSP_ISR_TX(void)
{
	unsigned int rxtemp;
   	if (SPCR && 2)
		rxtemp = DRR;
#if 0
	if (SPCR && 0x20000)	
		DXR = rxtemp;
#endif
}


interrupt void McBSP_ISR_RX(void)
{

	if (SPCR && 0x20000)	
		DXR = tempRX;
}





/* initilize EDMA controller 0 shadow region 1 for McBSP.*/

#define CC0_DRAE1 		*(unsigned int *)0x01c00348
#define CC0_EMCR  		*(unsigned int *)0x01c00308
#define CC0_CCERRCLR 	*(unsigned int *)0x01c0031c

#define CC0_SR1_EER 	*(unsigned int *)0x01c02220  //Event enable
#define CC0_SR1_EESR 	*(unsigned int *)0x01c02230  //Event enable
#define CC0_SR1_IER 	*(unsigned int *)0x01c02250  // Interrupt enable
#define CC0_SR1_IESR 	*(unsigned int *)0x01c02260  // Interrupt enable
#define CC0_SR1_IPR 	*(unsigned int *)0x01c02268  // Interrupt pending 
#define CC0_SR1_ICR 	*(unsigned int *)0x01c02270  // interrupt clear


unsigned short TXBUFF0[4]={0};
unsigned short TXBUFF1[4]={0};

EDMACC0_init()
{

	CC0_DRAE1 = 0xFFFFFFFF;
	CC0_EMCR  = 0xffffffff;
	CC0_CCERRCLR = 0xFFFFFFFF;


	//Parameter 3
	*(unsigned int *)0x01c04060 =  (1<<20 | 3<<12 | 0<<3 |0<<2| 0<<1|0<<0);  //OPT
	*(unsigned int *)0x01c04064 =   (unsigned int)TXBUFF0;                  //SRC
	*(unsigned int *)0x01c04068 =  (4<<16|2);    //A_B_CNT
	*(unsigned int *)0x01c0406c =   0x01D10004;   //DST
	*(unsigned int *)0x01c04070 =  (0<<16|2);     //SRC_DST_INDEX
	*(unsigned int *)0x01c04074 =   0x880;        //LINK_BCNT
	*(unsigned int *)0x01c04078 =   0;            //SRC_DST_CIDX
	*(unsigned int *)0x01c0407c =   1;            //CCNT

	//Parameter 68
	*(unsigned int *)0x01c04880 = (1<<20 | 3<<12 | 0<<3 |0<<2| 0<<1|0<<0);
	*(unsigned int *)0x01c04884 =  (unsigned int)TXBUFF1;
	*(unsigned int *)0x01c04888 = (4<<16|2);
	*(unsigned int *)0x01c0488c =  0x01D10004;
	*(unsigned int *)0x01c04890 = (0<<16|2);
	*(unsigned int *)0x01c04894 = 0x8a0;
	*(unsigned int *)0x01c04898 = 0;
	*(unsigned int *)0x01c0489c = 1;

	//Parameter 69
	*(unsigned int *)0x01c048a0 = (1<<20 | 3<<12 | 0<<3 |0<<2| 0<<1|0<<0);
	*(unsigned int *)0x01c048a4 =(unsigned int)TXBUFF0;
	*(unsigned int *)0x01c048a8 = ((4<<16)|2);
	*(unsigned int *)0x01c048ac = 0x01D10004;
	*(unsigned int *)0x01c048b0 = ((0<<16)|2); 
	*(unsigned int *)0x01c048b4 = 0x880;
	*(unsigned int *)0x01c048b8 = 0;
	*(unsigned int *)0x01c048bc = 1;

	CC0_SR1_EESR = 0x8; //McBSP0 transmit event
	CC0_SR1_IESR = 0x8; //McBSP0 transmit interrupt
}


interrupt void EDMA_ISR()
{
	unsigned int temp;
	temp = CC0_SR1_IPR;
	CC0_SR1_ICR = temp;

}